<?php
/**
 * @version $Id: dbqbarter.php 1542 2012-05-03 05:39:49Z roosit $
 * @package Abricos
 * @subpackage Job
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin <roosit@abricos.org>
 */

class JobBarterQuery {
	
	public static function CategoryBarterCount(Ab_Database $db){
		$sql = "
			SELECT p.categoryid as id, count(*) as c
			FROM ".$db->prefix."job_barter p
			WHERE p.pubdate>0 AND p.isdraft=0
			GROUP BY p.categoryid
		";
		return $db->query_read($sql);
	}
	
	const PRJLIMIT = 15;
	
	const BarterFields = "
		p.barterid as id, 
		p.categoryid as catid,
		p.userid as uid,
		p.title as tl,
		p.descript as dsc,
		p.pubdate as pdt,
		p.answercount as acnt
	";
	
	public static function BarterListWhere($userid=0, $catid='', $bartid=0){
		$where = "(p.pubdate>0 AND p.isdraft=0)";
		
		if ($userid > 0){
			$where .= " OR (p.userid=".bkint($userid).")";
		}
		
		if (is_string($catid) && $catid == 'my'){
			$where = "(".$where.") AND p.userid=".bkint($userid)." ";
		}else if (intval($catid)>0){
			$where = "(".$where.") AND p.categoryid=".bkint($catid)." ";
		}
		$where = "(".$where.") AND p.deldate=0 AND p.isspam=0";
		if ($bartid>0){
			$where = "(".$where.") AND p.barterid=".bkint($bartid)." ";
		}
		return $where;
	}
	
	public static function BarterList(Ab_Database $db, $userid=0, $catid='', $next=0, $bartid=0){

		$fields = JobBarterQuery::BarterFields;
		$limit = "".bkint($from).",".bkint($limit)."";
		$where = JobBarterQuery::BarterListWhere($userid, $catid, $bartid);
		$order = "p.pubdate DESC";
		
		if ($userid > 0){ // запрашивает автор
			$fields .= ",
				p.isdraft as dft
			";
			$order = "p.isdraft DESC,".$order;
		}
		
		if ($bartid>0){
			$limit = "1";
		}
		
		$sql = "
			SELECT ".$fields."
			FROM ".$db->prefix."job_barter p
			WHERE ".$where." 
			ORDER BY ".$order."
			LIMIT ".(bkint($next)).", ".JobBarterQuery::PRJLIMIT."
		";
		return $bartid > 0 ? $db->query_first($sql) : $db->query_read($sql);
	}
	
	public static function BarterListCount(Ab_Database $db, $userid=0, $catid=''){
		$where = JobBarterQuery::BarterListWhere($userid, $catid, 0);
		$sql = "
			SELECT count(*) as cnt
			FROM ".$db->prefix."job_barter p
			WHERE ".$where." 
		";
		$row = $db->query_first($sql);
		return $row['cnt'];
	}
	
	public static function MyBarterAppend(Ab_Database $db, $userid, $p){
		$sql = "
			INSERT INTO ".$db->prefix."job_barter (
				categoryid, userid, title, descript,
				isdraft, pubdate, upddate, dateline
			) VALUES (
				".bkint($p->catid).",
				".bkint($userid).",
				'".bkstr($p->tl)."',
				'".bkstr($p->dsc)."',
				
				".intval($p->dft).",
				".intval($p->pdt).",
				".TIMENOW.",
				".TIMENOW."
			)
		";
		$db->query_write($sql);
		return $db->insert_id();
	}
	
	public static function MyBarterUpdate(Ab_Database $db, $userid, $p){
		$sql = "
			UPDATE ".$db->prefix."job_barter
			SET 
				categoryid=".bkint($p->catid).",
				title='".bkstr($p->tl)."',
				descript='".bkstr($p->dsc)."',
				
				agreement=".intval($p->agr).",
				cost=".intval($p->cst).",
				currencytype=".intval($p->cctp).",
				priceby=".intval($p->pctp).",
				
				isdraft=".intval($p->dft).",
				pubdate=".$p->pdt.",
				upddate=".TIMENOW."
			WHERE barterid=".bkint($p->id)." AND userid=".bkint($userid)."
		";
		$db->query_write($sql);
	}
	
	public static function MyBarterSetRemoved(Ab_Database $db, $userid, $bartid){
		$sql = "
			UPDATE ".$db->prefix."job_barter
				deldate=".TIMENOW."
			WHERE barterid=".bkint($bartid)." AND userid=".bkint($userid)."
		";
		$db->query_write($sql);
		return $db->affected_rows();
	}
	
	public static function BarterUserInit(Ab_Database $db, $bartid, $userid){
		if (empty($userid) || empty($bartid)){ return; }
		$sql = "
			INSERT IGNORE INTO ".$db->prefix."job_barteruser (barterid, userid, dateline) VALUES (
				".bkint($bartid).", ".bkint($userid).", ".TIMENOW."
			)
		";
		$db->query_write($sql);
	}
	
	public static function BarterUserUpdateLastView(Ab_Database $db, $bartid, $userid){
		if (empty($userid) || empty($bartid)){ return; }
		JobBarterQuery::BarterUserInit($db, $bartid, $userid);
		$sql = "
			UPDATE ".$db->prefix."job_barteruser
			SET lastview=".TIMENOW."
			WHERE barterid=".bkint($bartid)." AND userid=".bkint($userid)."
			LIMIT 1
		";
		$db->query_write($sql);
	}
	
	public static function BarterAnswerCountUpdate(Ab_Database $db, $bartid){
		$sql = "
			UPDATE ".$db->prefix."job_barter
			SET answercount=(
				SELECT count(*) as cnt
				FROM ".$db->prefix."job_banswer
				WHERE barterid=".bkint($bartid)."
				GROUP BY barterid
			)
			WHERE barterid=".bkint($bartid)."
			LIMIT 1
		";
		$db->query_write($sql);
	}
	
	public static function AnswerCommentCountUpdate(Ab_Database $db, $ctid){
		$sql = "
			UPDATE ".$db->prefix."job_banswer
			SET cmtcount=(
				SELECT count(*) as cnt
				FROM ".$db->prefix."cmt_comment
				WHERE contentid=".bkint($ctid)."
				GROUP BY contentid
			)
			WHERE contentid=".bkint($ctid)."
			LIMIT 1
		";
		$db->query_write($sql);
	}
	
	public static function AnswerList(Ab_Database $db, $bartid, $userid){
		$bartid = bkint($bartid);
		$userid = bkint($userid);
		$sql = "
			SELECT 
				a.banswerid as id,
				a.userid as uid,
				a.contentid as ctid,
				c.body as dsc,
				a.btime as btm,
				a.etime as etm,
				a.ttime as ttm,
				a.isprivate as prv,
				a.dateline as dl,
				a.cmtcount as ccnt
			FROM ".$db->prefix."job_banswer a
			INNER JOIN ".$db->prefix."job_barter p ON p.barterid=a.barterid
			INNER JOIN ".$db->prefix."content c ON a.contentid=c.contentid
			WHERE 
				(p.barterid=".$bartid." AND p.isdraft=0 AND p.pubdate>0) 
				AND 
				(
					(a.isprivate=0)
					OR 
					(a.isprivate>0 AND (a.userid=".$userid." OR p.userid=".$userid."))
				)
		";
		return $db->query_read($sql);
	}
	
	public static function AnswerAppend(Ab_Database $db, $bartid, $userid, $a){
		$contentid = Ab_CoreQuery::ContentAppend($db, $a->dsc, 'job');
		
		$sql = "
			INSERT INTO ".$db->prefix."job_banswer
			(barterid, userid, contentid, isprivate, dateline) VALUES (
				".bkint($bartid).",
				".bkint($userid).",
				".$contentid.",
				".bkint($a->prv).",
				".TIMENOW."
			)
		";
		$db->query_write($sql);
		return $db->insert_id();
	}

}

// приватные запросы к бд, для служебных целей
class JobBarterQueryPrivate {
	
	public static function Barter(Ab_Database $db, $bartid){
		$sql = "
			SELECT *
			FROM ".$db->prefix."job_barter
			WHERE barterid=".bkint($bartid)."
			LIMIT 1
		";
		return $db->query_first($sql);
	}
	
	public static function Answer(Ab_Database $db, $bartid, $userid){
		$sql = "
			SELECT *
			FROM ".$db->prefix."job_banswer
			WHERE barterid=".bkint($bartid)." AND userid=".bkint($userid)."
			LIMIT 1
		";
		return $db->query_first($sql);
	}
	
}

?>