<?php
/**
 * @version $Id: dbqproject.php 1542 2012-05-03 05:39:49Z roosit $
 * @package Abricos
 * @subpackage Job
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin <roosit@abricos.org>
 */

class JobProjectQuery {
	
	public static function CategoryProjectCount(Ab_Database $db){
		$sql = "
			SELECT p.categoryid as id, count(*) as c
			FROM ".$db->prefix."job_project p
			WHERE p.pubdate>0 AND p.isdraft=0
			GROUP BY p.categoryid
		";
		return $db->query_read($sql);
	}
	
	const PRJLIMIT = 15;
	
	const ProjectFields = "
		p.projectid as id, 
		p.categoryid as catid,
		p.userid as uid,
		p.title as tl,
		p.descript as dsc,
		p.agreement as agr,
		p.cost as cst,
		p.currencytype as cctp,
		p.priceby as pctp,
		p.pubdate as pdt,
		p.answercount as acnt
	";
	
	public static function ProjectListWhere($userid=0, $catid='', $projid=0){
		$where = "(p.pubdate>0 AND p.isdraft=0)";
		
		if ($userid > 0){
			$where .= " OR (p.userid=".bkint($userid).")";
		}
		
		if (is_string($catid) && $catid == 'my'){
			$where = "(".$where.") AND p.userid=".bkint($userid)." ";
		}else if (intval($catid)>0){
			$where = "(".$where.") AND p.categoryid=".bkint($catid)." ";
		}
		$where = "(".$where.") AND p.deldate=0 AND p.isspam=0";
		if ($projid>0){
			$where = "(".$where.") AND p.projectid=".bkint($projid)." ";
		}
		return $where;
	}
	
	public static function ProjectList(Ab_Database $db, $userid=0, $catid='', $next=0, $projid=0){

		$fields = JobProjectQuery::ProjectFields;
		$limit = "".bkint($from).",".bkint($limit)."";
		$where = JobProjectQuery::ProjectListWhere($userid, $catid, $projid);
		$order = "p.pubdate DESC";
		
		if ($userid > 0){ // запрашивает автор
			$fields .= ",
				p.isdraft as dft
			";
			$order = "p.isdraft DESC,".$order;
		}
		
		if ($projid>0){
			$limit = "1";
		}
		
		$sql = "
			SELECT ".$fields."
			FROM ".$db->prefix."job_project p
			WHERE ".$where." 
			ORDER BY ".$order."
			LIMIT ".(bkint($next)).", ".JobProjectQuery::PRJLIMIT."
		";
		return $projid > 0 ? $db->query_first($sql) : $db->query_read($sql);
	}
	
	public static function ProjectListCount(Ab_Database $db, $userid=0, $catid=''){
		$where = JobProjectQuery::ProjectListWhere($userid, $catid, 0);
		$sql = "
			SELECT count(*) as cnt
			FROM ".$db->prefix."job_project p
			WHERE ".$where." 
		";
		$row = $db->query_first($sql);
		return $row['cnt'];
	}
	
	public static function MyProjectAppend(Ab_Database $db, $userid, $p){
		$sql = "
			INSERT INTO ".$db->prefix."job_project (
				categoryid, userid, title, descript,
				agreement, cost, currencytype, priceby, 
				isdraft, pubdate, upddate, dateline
			) VALUES (
				".bkint($p->catid).",
				".bkint($userid).",
				'".bkstr($p->tl)."',
				'".bkstr($p->dsc)."',
				
				".intval($p->agr).",
				".intval($p->cst).",
				".intval($p->cctp).",
				".intval($p->pctp).",
				
				".intval($p->dft).",
				".intval($p->pdt).",
				".TIMENOW.",
				".TIMENOW."
			)
		";
		$db->query_write($sql);
		return $db->insert_id();
	}
	
	public static function MyProjectUpdate(Ab_Database $db, $userid, $p){
		$sql = "
			UPDATE ".$db->prefix."job_project
			SET 
				categoryid=".bkint($p->catid).",
				title='".bkstr($p->tl)."',
				descript='".bkstr($p->dsc)."',
				
				agreement=".intval($p->agr).",
				cost=".intval($p->cst).",
				currencytype=".intval($p->cctp).",
				priceby=".intval($p->pctp).",
				
				isdraft=".intval($p->dft).",
				pubdate=".$p->pdt.",
				upddate=".TIMENOW."
			WHERE projectid=".bkint($p->id)." AND userid=".bkint($userid)."
		";
		$db->query_write($sql);
	}
	
	public static function MyProjectSetRemoved(Ab_Database $db, $userid, $projid){
		$sql = "
			UPDATE ".$db->prefix."job_project
				deldate=".TIMENOW."
			WHERE projectid=".bkint($projid)." AND userid=".bkint($userid)."
		";
		$db->query_write($sql);
		return $db->affected_rows();
	}
	
	public static function ProjectUserInit(Ab_Database $db, $projid, $userid){
		if (empty($userid) || empty($projid)){ return; }
		$sql = "
			INSERT IGNORE INTO ".$db->prefix."job_projectuser (projectid, userid, dateline) VALUES (
				".bkint($projid).", ".bkint($userid).", ".TIMENOW."
			)
		";
		$db->query_write($sql);
	}
	
	public static function ProjectUserUpdateLastView(Ab_Database $db, $projid, $userid){
		if (empty($userid) || empty($projid)){ return; }
		JobProjectQuery::ProjectUserInit($db, $projid, $userid);
		$sql = "
			UPDATE ".$db->prefix."job_projectuser
			SET lastview=".TIMENOW."
			WHERE projectid=".bkint($projid)." AND userid=".bkint($userid)."
			LIMIT 1
		";
		$db->query_write($sql);
	}
	
	public static function ProjectAnswerCountUpdate(Ab_Database $db, $projid){
		$sql = "
			UPDATE ".$db->prefix."job_project
			SET answercount=(
				SELECT count(*) as cnt
				FROM ".$db->prefix."job_panswer
				WHERE projectid=".bkint($projid)."
				GROUP BY projectid
			)
			WHERE projectid=".bkint($projid)."
			LIMIT 1
		";
		$db->query_write($sql);
	}
	
	public static function AnswerCommentCountUpdate(Ab_Database $db, $ctid){
		$sql = "
			UPDATE ".$db->prefix."job_panswer
			SET cmtcount=(
				SELECT count(*) as cnt
				FROM ".$db->prefix."cmt_comment
				WHERE contentid=".bkint($ctid)."
				GROUP BY contentid
			)
			WHERE contentid=".bkint($ctid)."
			LIMIT 1
		";
		$db->query_write($sql);
	}
	
	public static function AnswerList(Ab_Database $db, $projid, $userid){
		$projid = bkint($projid);
		$userid = bkint($userid);
		$sql = "
			SELECT 
				a.panswerid as id,
				a.userid as uid,
				a.contentid as ctid,
				c.body as dsc,
				a.bcost,
				a.ecost,
				a.currencytype as ctp,
				a.btime as btm,
				a.etime as etm,
				a.ttime as ttm,
				a.isprivate as prv,
				a.dateline as dl,
				a.cmtcount as ccnt
			FROM ".$db->prefix."job_panswer a
			INNER JOIN ".$db->prefix."job_project p ON p.projectid=a.projectid
			INNER JOIN ".$db->prefix."content c ON a.contentid=c.contentid
			WHERE 
				(p.projectid=".$projid." AND p.isdraft=0 AND p.pubdate>0) 
				AND 
				(
					(a.isprivate=0)
					OR 
					(a.isprivate>0 AND (a.userid=".$userid." OR p.userid=".$userid."))
				)
		";
		return $db->query_read($sql);
	}
	
	public static function AnswerAppend(Ab_Database $db, $projid, $userid, $a){
		$contentid = Ab_CoreQuery::ContentAppend($db, $a->dsc, 'job');
		
		$sql = "
			INSERT INTO ".$db->prefix."job_panswer
			(projectid, userid, contentid, bcost, ecost, currencytype, 
				btime, etime, ttime, isprivate, dateline) VALUES (
				".bkint($projid).",
				".bkint($userid).",
				".$contentid.",
				".bkint($a->bcost).",
				".bkint($a->ecost).",
				".bkint($a->ctp).",
				".bkint($a->btm).",
				".bkint($a->etm).",
				".bkint($a->ttm).",
				".bkint($a->prv).",
				".TIMENOW."
			)
		";
		$db->query_write($sql);
		return $db->insert_id();
	}

}

// приватные запросы к бд, для служебных целей
class JobProjectQueryPrivate {
	
	public static function Project(Ab_Database $db, $projid){
		$sql = "
			SELECT *
			FROM ".$db->prefix."job_project
			WHERE projectid=".bkint($projid)."
			LIMIT 1
		";
		return $db->query_first($sql);
	}
	
	public static function Answer(Ab_Database $db, $projid, $userid){
		$sql = "
			SELECT *
			FROM ".$db->prefix."job_panswer
			WHERE projectid=".bkint($projid)." AND userid=".bkint($userid)." 
			LIMIT 1
		";
		return $db->query_first($sql);
	}
	
}

?>