<?php
/**
 * @version $Id: dbquery.php 1542 2012-05-03 05:39:49Z roosit $
 * @package Abricos
 * @subpackage Job
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin <roosit@abricos.org>
 */

class JobQuery {
	
	public static function UserListByIds(Ab_Database $db, $ids){
		$wh = array("u.userid=0");
		foreach($ids as $id){
			array_push($wh, "u.userid=".intval($id));
		}
		$sql = "
			SELECT
				u.userid as id,
				u.username as unm,
				u.firstname as fnm,
				u.lastname as lnm,
				u.avatar as avt 
			FROM ".$db->prefix."user u
			WHERE ".implode(" OR ", $wh)."
		";
		return $db->query_read($sql);
	}
	
	public static function ScopeUpdate(Ab_Database $db, $userid, $sc){
		$sql = "
			UPDATE ".$db->prefix."job_scope 
			SET 
				descript='".bkstr($sc->dsc)."',
				upddate=".TIMENOW.",
				deldate=0
			WHERE userid=".bkint($userid)." AND categoryid=".bkint($sc->catid)."
			LIMIT 1
		";
		$db->query_write($sql);
	}
	
	public static function ScopeRemove(Ab_Database $db, $userid, $sc){
		$sql = "
			UPDATE ".$db->prefix."job_scope 
			SET deldate=".TIMENOW."
			WHERE userid=".bkint($userid)." AND categoryid=".bkint($sc->catid)."
			LIMIT 1
		";
		$db->query_write($sql);
	}
	
	public static function ScopeAppend(Ab_Database $db, $userid, $sc){
		$sql = "
			INSERT INTO ".$db->prefix."job_scope 
			(categoryid, userid, descript, dateline, upddate) VALUES (
				".bkint($sc->catid).",
				".bkint($userid).",
				'".bkstr($sc->dsc)."',
				".TIMENOW.",
				".TIMENOW."
			)
		";
		$db->query_write($sql);
		return $db->insert_id();
	}
	
	public static function ScopeList(Ab_Database $db, $userid, $withdel = false){
		$where = " AND deldate=0";
		$fields = "";
		if ($withdel){
			$where = "";
			$fields = ",
				dateline as dl,
				upddate as ud,
				deldate as dd
			";
		}
		$sql = "
			SELECT 
				scopeid as id,
				categoryid as catid,
				descript as dsc
				".$fields."
			FROM ".$db->prefix."job_scope
			WHERE userid=".bkint($userid)." ".$where."
		";
		return $db->query_read($sql);
	}
	
	public static function CategoryList(Ab_Database $db, $catid=0){
		$where = "";
		if ($catid > 0){
			$where = " AND categoryid=".bkint($catid)." LIMIT 1 ";
		}
		$sql = "
			SELECT 
				categoryid as id,
				parentcategoryid as pid,
				title as tl,
				onlyproject as p,
				onlybarter as b,
				ordw as o
			FROM ".$db->prefix."job_category
			WHERE deldate=0 ".$where."
			ORDER BY ordw DESC, title
		";
		return $catid>0 ? $db->query_first($sql) : $db->query_read($sql);
	}
		
	public static function CategoryAppend(Ab_Database $db, $pid, $title, $descript = "", $onlyProject = 0, $onlyBarter = 0, $ordw = 0){
		$sql = "
			INSERT INTO ".$db->prefix."job_category 
			(parentcategoryid, title, descript, onlyproject, onlybarter, ordw, dateline) VALUES (
				".bkint($pid).", 
				'".bkstr($title)."', 
				'".bkstr($descript)."', 
				".bkint($onlyProject).", 
				".bkint($onlyBarter).", 
				".bkint($ordw).", 
				".TIMENOW."
			)
		";
		$db->query_write($sql);
		return $db->insert_id();
	}
	
	public static function CategoryUpdate(Ab_Database $db, $id, $title, $ordw = 0){
		$sql = "
			UPDATE ".$db->prefix."job_category
			SET 
				title='".bkstr($title)."',
				ordw=".bkint($ordw)."
			WHERE categoryid=".bkint($id)."
		";
		$db->query_write($sql);
	}
	
	public static function CategoryRemove(Ab_Database $db, $id){
		$sql = "
			UPDATE ".$db->prefix."job_category
			SET deldate=".TIMENOW."
			WHERE parentcategoryid=".bkint($id)."
		";
		$db->query_write($sql);
		
		$sql = "
			UPDATE ".$db->prefix."job_category
			SET deldate=".TIMENOW."
			WHERE categoryid=".bkint($id)."
			LIMIT 1
		";
		$db->query_write($sql);
	}
	
	public static function ProjectCountList(Ab_Database $db){
		$sql = "
			SELECT p.categoryid as id, count(*) as c
			FROM ".$db->prefix."job_project p
			WHERE p.pubdate>0 AND p.isdraft=0
			GROUP BY p.categoryid
		";
		return $db->query_read($sql);
	}
	
	const PRJLIMIT = 15;
	
	const ProjectFields = "
		p.projectid as id, 
		p.categoryid as catid,
		p.userid as uid,
		p.title as tl,
		p.descript as dsc,
		p.agreement as agr,
		p.cost as cst,
		p.currencytype as cctp,
		p.priceby as pctp,
		p.pubdate as pdt
	";
	
	private static function ProjectList(Ab_Database $db, $userid=0, $catid=0, $page=1, $projid=0){
		$limit = JobQuery::PRJLIMIT;
		$from = $limit * (max($page, 1) - 1);

		$fields = JobQuery::ProjectFields;
		$limit = "".bkint($from).",".bkint($limit)."";
		$where = "(p.pubdate>0 AND p.isdraft=0)";
		$order = "p.pubdate DESC";
		
		if ($userid > 0){ // запрашивает автор
			$fields .= ",
				p.isdraft as dft
			";
			$where .= " OR (p.userid=".bkint($userid).")";
			$order = "p.isdraft DESC,".$order;
		}
		if ($catid>0){
			$where = "(".$where.") AND p.categoryid=".bkint($catid)." ";
		}
		$where = "(".$where.") AND p.deldate=0 AND p.isspam=0";
		if ($projid>0){
			$where = "(".$where.") AND p.projectid=".bkint($projid)." ";
			$limit = "1";
		}
		
		$sql = "
			SELECT ".$fields."
			FROM ".$db->prefix."job_project p
			WHERE ".$where."
			ORDER BY ".$order."
			LIMIT ".$limit."
		";
		return $projid > 0 ? $db->query_first($sql) : $db->query_read($sql);
	}
	
	public static function GuestProjectList(Ab_Database $db, $catid=0, $page=1, $projid=0){
		return JobQuery::ProjectList($db, 0, $catid, $page, $projid);
	}
	
	public static function MyProjectList(Ab_Database $db, $userid, $catid=0, $page=1, $projid=0){
		return JobQuery::ProjectList($db, $userid, $catid, $page, $projid);
	}
	
	public static function MyProjectAppend(Ab_Database $db, $userid, $p){
		$sql = "
			INSERT INTO ".$db->prefix."job_project (
				categoryid, userid, title, descript,
				agreement, cost, currencytype, priceby, 
				isdraft, pubdate, upddate, dateline
			) VALUES (
				".bkint($p->catid).",
				".bkint($userid).",
				'".bkstr($p->tl)."',
				'".bkstr($p->dsc)."',
				
				".intval($p->agr).",
				".intval($p->cst).",
				".intval($p->cctp).",
				".intval($p->pctp).",
				
				".intval($p->dft).",
				".intval($p->pdt).",
				".TIMENOW.",
				".TIMENOW."
			)
		";
		$db->query_write($sql);
		return $db->insert_id();
	}
	
	public static function MyProjectUpdate(Ab_Database $db, $userid, $p){
		$sql = "
			UPDATE ".$db->prefix."job_project
			SET 
				categoryid=".bkint($p->catid).",
				title='".bkstr($p->tl)."',
				descript='".bkstr($p->dsc)."',
				
				agreement=".intval($p->agr).",
				cost=".intval($p->cst).",
				currencytype=".intval($p->cctp).",
				priceby=".intval($p->pctp).",
				
				isdraft=".intval($p->dft).",
				pubdate=".$p->pdt.",
				upddate=".TIMENOW."
			WHERE projectid=".bkint($p->id)." AND userid=".bkint($userid)."
		";
		$db->query_write($sql);
	}
	
	public static function MyProjectSetRemoved(Ab_Database $db, $userid, $projid){
		$sql = "
			UPDATE ".$db->prefix."job_project
				deldate=".TIMENOW."
			WHERE projectid=".bkint($projid)." AND userid=".bkint($userid)."
		";
		$db->query_write($sql);
		return $db->affected_rows();
	}
	
}

// приватные запросы к бд, для служебных целей
class JobQueryPrivate {
	public static function ProjectAnswerByContentId(Ab_Database $db, $contentid){
		$sql = "
			SELECT 
				a.panswerid as id,
				p.projectid as pid,
				p.userid as author,
				a.isprivate,
				a.userid,
				a.cmtcount,
				'project' as rtype
			FROM ".$db->prefix."job_panswer a
			INNER JOIN ".$db->prefix."job_project p ON p.projectid=a.projectid
			WHERE a.contentid=".bkint($contentid)."
			LIMIT 1
		";
		return $db->query_first($sql);
	}
	public static function BarterAnswerByContentId(Ab_Database $db, $contentid){
		$sql = "
			SELECT 
				a2.banswerid as id,
				b.userid as author,
				b.barterid as pid,
				a2.isprivate,
				a2.userid,
				a2.cmtcount,
				'barter' as rtype
			FROM ".$db->prefix."job_banswer a2
			INNER JOIN ".$db->prefix."job_barter b ON b.barterid=a2.barterid
			WHERE a2.contentid=".bkint($contentid)."
			LIMIT 1
		";
		return $db->query_first($sql);
	}
	
}

?>