<?php
/**
 * @version $Id: manager.php 1542 2012-05-03 05:39:49Z roosit $
 * @package Abricos
 * @subpackage Job
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin <roosit@abricos.org>
 */

require_once 'dbquery.php';

/**
 * Управляющий класс модуля
 */
class JobManager extends Ab_ModuleManager {
	
	/**
	 * @var JobModule
	 */
	public $module = null;
	
	/**
	 * @var JobManager
	 */
	public static $instance = null;
	
	private $_manProject = null;
	private $_manBarter = null;
	private $_manExpro = null;
	
	private $_disableRoles = false;
	
	/**
	 * Конструктор
	 * @param JobModule $module
	 */
	public function __construct(JobModule $module){
		parent::__construct($module);
		
		JobManager::$instance = $this;
	}
	
	public function DisableRoles(){
		$this->_disableRoles = true;
	}
	
	public function EnableRoles(){
		$this->_disableRoles = false;
	}
	
	public function IsAdminRole(){
		if ($this->_disableRoles){ return true; }
		return $this->IsRoleEnable(JobAction::ADMIN);
	}
	
	public function IsProAccountRole(){
		if ($this->IsAdminRole()){ return true; }
		return $this->IsRoleEnable(JobAction::PROACCOUNT);
	}
	
	public function IsWriteRole(){
		if ($this->IsProAccountRole()){ return true; }
		return $this->IsRoleEnable(JobAction::WRITE);
	}
	
	public function IsViewRole(){
		if ($this->IsWriteRole()){ return true; }
		return $this->IsRoleEnable(JobAction::VIEW);
	}
	
	public function AJAX($d){
		if ($d->type == 'project'){
			return $this->GetProjectManager()->AJAX($d);
		}else if ($d->type == 'expro'){
			return $this->GetExproManager()->AJAX($d);
		}else if ($d->type == 'barter'){
			return $this->GetBarterManager()->AJAX($d);
		}else{
			switch($d->do){
				case 'home': return $this->Home($d->load);
				case 'categorylist': return $this->CategoryList();
				case 'categorysave': return $this->CategorySave($d);
				case 'categoryremove': return $this->CategoryRemove($d->id);
				
				case 'scopelist': return $this->ScopeList($d->userid);
				case 'scopesave': return $this->ScopeSave($d->list);
			}
		}
		return null;
	}
	
	/**
	 * @return JobExproManager
	 */
	public function GetExproManager(){
		if (is_null($this->_manExpro)){
			require_once 'manexpro.php';
			$this->_manExpro = new JobExproManager($this);
		}
		return $this->_manExpro;
	}

	/**
	 * @return JobProjectManager
	 */
	public function GetProjectManager(){
		if (is_null($this->_manProject)){
			require_once 'manproject.php';
			$this->_manProject = new JobProjectManager($this);
		}
		return $this->_manProject;
	}
	
	/**
	 * @return JobBarterManager
	 */
	public function GetBarterManager(){
		if (is_null($this->_manBarter)){
			require_once 'manbarter.php';
			$this->_manBarter = new JobBarterManager($this);
		}
		return $this->_manBarter;
	}
	
	public function ToArray($rows, &$userids = ""){
		$ret = array();
		while (($row = $this->db->fetch_array($rows))){
			array_push($ret, $row);
			if (is_array($userids)){
				$userids[$row['uid']] = $row['uid'];
			}
		}
		return $ret;
	}
	
	public function ToArrayId($rows, $field = "id"){
		$ret = array();
		while (($row = $this->db->fetch_array($rows))){
			$ret[$row[$field]] = $row;
		}
		return $ret;
	}
	
	public function Home($load){
		$cfg = new stdClass();
		$cfg->catid = 0;
		
		$ret = new stdClass();
		if ($load->ismain){
			$ret->category = $this->CategoryList();
		}
		if ($load->isproject){
			$ret->project = $this->GetProjectManager()->ProjectList($cfg);
		}
		if ($load->isbarter){
			$ret->barter = $this->GetBarterManager()->BarterList($cfg);
		}
		
		return $ret;
	}
	
	public function ScopeList($userid, $withdel = false, $isIdArray = false){
		if (!$this->IsViewRole()){ return null; }
		
		$rows = JobQuery::ScopeList($this->db, $userid, $withdel);
		return $isIdArray ? $this->ToArrayId($rows, "catid") : $this->ToArray($rows);
	}
	
	public function ScopeSave($list){
		if (!$this->IsWriteRole()){ return null; }
		
		$dblist = $this->ScopeList($this->userid, true, true);
		
		$utmf = Abricos::TextParser(true);
		$utmf->jevix->cfgSetAutoBrMode(true);

		foreach($list as $sc){
			if (!$this->CategoryCheck($sc->catid)){
				// попытки использовать не существующий каталог нужно пресекать
				// sleep(1);
				// return null;
				$sc->f = 'd';
			}
			$sc->dsc = $utmf->Parser($sc->dsc);
			if (empty($sc->id)){
				if (!empty($dblist[$sc->catid])){
					$sc->id = $dblist[$sc->catid]['id'];
					JobQuery::ScopeUpdate($this->db, $this->userid, $sc);
				}else{
					JobQuery::ScopeAppend($this->db, $this->userid, $sc);
				}
			}else{
				if ($sc->f == 'd'){
					JobQuery::ScopeRemove($this->db, $this->userid, $sc);
				}else{
					JobQuery::ScopeUpdate($this->db, $this->userid, $sc);
				}
			}
		}
		return $this->ScopeList($this->userid);
	}
	
	public function CategoryCheck($catid){
		$cats = $this->CategoryList(true);
		return !empty($cats[$catid]);
	}

	private $_catCache = null;
	private $_catCacheId = null;
	public function CategoryList($isIdArray = false){
		if (!$this->IsViewRole()){ return null; }
		
		if (!$isIdArray){
			if (is_null($this->_catCache)){
				$rows = JobQuery::CategoryList($this->db);
				$this->_catCache = $this->ToArray($rows);
			}
			return $this->_catCache;
		}else{
			if (is_null($this->_catCacheId)){
				$rows = JobQuery::CategoryList($this->db);
				$this->_catCacheId = $this->ToArrayId($rows);
			}
			return $this->_catCacheId;
		}
	}
	
	public function Category($catid){
		if (!$this->IsViewRole()){ return null; }
		return JobQuery::CategoryList($this->db, $catid);
	}
	
	public function CategorySave($d){
		if (!$this->IsAdminRole()){ return null; }
		
		if ($d->id > 0){
			JobQuery::CategoryUpdate($this->db, $d->id, $d->tl, $d->o);
		}else{
			JobQuery::CategoryAppend($this->db, $d->pid, $d->tl, "", 0, 0, $d->o);
		}
		
		return $this->CategoryList();
	}
	
	public function CategoryRemove($catid){
		if (!$this->IsAdminRole()){
			return null;
		}
		
		JobQuery::CategoryRemove($this->db, $catid);
		
		return $this->CategoryList();
	}

	public function UserNameBuild($user){
		$firstname = !empty($user['fnm']) ? $user['fnm'] : $user['firstname'];
		$lastname = !empty($user['lnm']) ? $user['lnm'] : $user['lastname'];
		$username = !empty($user['unm']) ? $user['unm'] : $user['username'];
		return (!empty($firstname) && !empty($lastname)) ? $firstname." ".$lastname : $username;
	}
	
	////////////////////////////// комментарии /////////////////////////////

	private $_cmtinfo = null;
	public function IsCommentAppend($contentid){
		if (!$this->IsWriteRole()){ return null; }
		
		$a = JobQueryPrivate::ProjectAnswerByContentId($this->db, $contentid);
		if (empty($a)){
			$a = JobQueryPrivate::BarterAnswerByContentId($this->db, $contentid);
			if (empty($a)){
				return false;
			} 
		}
		
		// доступ к комментариям только у авторов проекта и приложения
		if ($a['userid'] == $this->userid || $a['author'] == $this->userid){ 
			$this->_cmtinfo = $a;
			return true; 
		}
	
		return false;
	}
	
	
	public function IsCommentList($contentid){
		return $this->IsCommentAppend($contentid);
	}
	
	/**
	* Отправить уведомление о новом комментарии
	*
	* @param object $data
	*/
	public function CommentSendNotify($data){
		if (!$this->IsWriteRole()){ return; }
		
		$a = $this->_cmtinfo;
		if (is_null($a)){ return false; }
		
		if ($a['rtype'] == 'project'){
			$this->GetProjectManager()->CommentSendNotify($a, $data);
		}else if ($a['rtype'] == 'barter'){
			$this->GetBarterManager()->CommentSendNotify($a, $data);
		}
		
		return false;
	}
	
}

?>