<?php
/**
 * @version $Id: manbarter.php 1542 2012-05-03 05:39:49Z roosit $
 * @package Abricos
 * @subpackage Job
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin <roosit@abricos.org>
 */

require_once 'dbqbarter.php';

/**
 * Управляющий класс модуля
 */
class JobBarterManager {
	
	/**
	 * @var JobManager
	 */
	public $manager = null;
	
	/**
	 * @var Ab_Database
	 */
	public $db = null;
	
	/**
	 * User
	 * @var User
	 */
	public $user = null;
	public $userid = 0;
	

	public function __construct(JobManager $manager){
		$this->manager = $manager;
		$this->user = $manager->user;
		$this->userid = $manager->userid;
		$this->db = $manager->db;
	}
	
	public function AJAX($d){
		switch($d->do){
			case 'bartersave':		return $this->MyBarterSave($d);
			case 'barterlist':		return $this->BarterList($d);
			case 'barter':			return $this->Barter($d->bartid);
			case 'answerappend':	return $this->AnswerAppend($d->bartid, $d->answer);
		}
		return null;
	}
	
	public function CategoryBarterCount(){
		if (!$this->manager->IsViewRole()){ return null; }

		$rows = JobBarterQuery::CategoryBarterCount($this->db);
		
		$ret = array();
		while (($row = $this->db->fetch_array($rows))){
			$ret[$row['id']] = intval($row['c']);
		}
		return $ret;
	}
	
	public function BarterList($cfg){
		if (!$this->manager->IsViewRole()){ return null; }
		
		$next = intval($cfg->next);
		$rows = JobBarterQuery::BarterList($this->db, $this->userid, $cfg->catid, $next);
		
		$users = array();
		$ret = new stdClass();
		$ret->list = $this->manager->ToArray($rows, $users);
		
		if ($next == 0){
			$ret->pcnt = $this->CategoryBarterCount();
			$ret->fcnt = JobBarterQuery::BarterListCount($this->db, $this->userid, $cfg->catid);
		}
		
		$rows = JobQuery::UserListByIds($this->db, $users);
		$ret->users = $this->manager->ToArray($rows);
		
		return $ret;
	}
	
	public function Barter($bartid){
		if (!$this->manager->IsViewRole() || empty($bartid)){ return null; }

		$ret = new stdClass();

		$ret->p = JobBarterQuery::BarterList($this->db, $this->userid, $catid, 0, $bartid);
		$ret->pcnt = $this->CategoryBarterCount();
		
		$users = array();
		$ret->p['answers'] = $this->AnswerList($bartid, $users);
		$users[$ret->p['uid']] = $ret->p['uid'] ;
		
		$rows = JobQuery::UserListByIds($this->db, $users);
		$ret->users = $this->manager->ToArray($rows);
		
		if ($this->userid > 0){
			JobBarterQuery::BarterUserUpdateLastView($this->db, $bartid, $this->userid);
		}
		
		return $ret;
	}
	
	public function AnswerList($bartid, &$users = ""){
		if (!$this->manager->IsViewRole()){ return null; }
		$rows = JobBarterQuery::AnswerList($this->db, $bartid, $this->userid);
		return $this->manager->ToArray($rows, $users);
	}
	
	public function AnswerAppend($bartid, $ans){
		$ret = new stdClass();
		$ret->err  = null;
		$ret->ansid = 0;
		$ret->barter = null;
		
		if (!$this->manager->IsWriteRole()){ 
			$ret->err = 1; 
			return $ret; 
		}
		
		$utmf = Abricos::TextParser(true);
		$utm  = Abricos::TextParser(true);
		$utm->jevix->cfgSetAutoBrMode(true);
		
		$cdesc = trim($utmf->Parser($ans->dsc));
		if (empty($cdesc)){
			$ret->err = 2; 
			return $ret; 
		}
		$ans->dsc = $utm->Parser($ans->dsc);
		
		
		$dbBart = JobBarterQueryPrivate::Barter($this->db, $bartid);
		if (
				empty($dbBart)
				|| $dbBart['deldate']>0 // проект удален
				|| $dbBart['userid'] == $this->userid // ответ на своей проект
				|| $dbBart['isdraft']>0 // на черновик отвечать нельзя
		){
			$ret->err = 3;
			return $ret;
		}

		$dbAns = JobBarterQueryPrivate::Answer($this->db, $bartid, $this->userid);
		if (!empty($bdAns)){ // можно оставить только одно предложение
			$ret->err = 4;
			return $ret;
		}
		
		$aid = JobBarterQuery::AnswerAppend($this->db, $bartid, $this->userid, $ans);
		$ret->err  = $error;
		$ret->ansid = $aid;
		$ret->barter = $this->Barter($bartid);
		
		JobBarterQuery::BarterAnswerCountUpdate($this->db, $bartid);
		
		// уведомить автора
		$autor = UserQuery::User($this->db, $dbBart['userid']);
		$email = $autor['email'];
		
		$brick = Brick::$builder->LoadBrickS('job', 'emailbarter', null, null);
		$v = &$brick->param->var;
		$host = $_SERVER['HTTP_HOST'] ? $_SERVER['HTTP_HOST'] : $_ENV['HTTP_HOST'];
		$plnk = "http://".$host."/bos/#app=job/wspace/ws/barter/BarterWorkspaceWidget/view/".$bartid."/";
		
		$subject = Brick::ReplaceVarByData($v['answersubject'], array(
			"tl" => $dbBart['title']
		));
		$body = Brick::ReplaceVarByData($v['answerbody'], array(
			"unm" => $this->manager->UserNameBuild($this->user->info),
			"plnk" => $plnk,
			"tl" => $dbBart['title'],
			"sitename" => Brick::$builder->phrase->Get('sys', 'site_name'),
			"anstext" => $ans->dsc
		));
		Abricos::Notify()->SendMail($email, $subject, $body);
		
		return $ret;
	}
	
	public function CommentSendNotify($ansInfo, $data){
	
		// данные по комментарию:
		// $data->id	- идентификатор комментария
		// $data->pid	- идентификатор родительского комментария
		// $data->uid	- пользователь оставивший комментарий
		// $data->bd	- текст комментария
		// $data->cid	- идентификатор контента
	
		JobBarterQuery::AnswerCommentCountUpdate($this->db, $data->cid);
	
		$bartid = $ansInfo['pid'];
	
		$bdBart = JobBarterQueryPrivate::Barter($this->db, $bartid);
		$bdAns = JobBarterQueryPrivate::Answer($this->db, $bartid, $ansInfo['userid']);
	
		// кому будет адресовано уведомление
		$userid = $this->userid == $bdBart['userid'] ? $bdAns['userid'] : $bdBart['userid'];
		$user = UserQuery::User($this->db, $userid);
		$email = $user['email'];
	
		$brick = Brick::$builder->LoadBrickS('job', 'emailbarter', null, null);
		$v = &$brick->param->var;
		$host = $_SERVER['HTTP_HOST'] ? $_SERVER['HTTP_HOST'] : $_ENV['HTTP_HOST'];
		$plnk = "http://".$host."/bos/#app=job/wspace/ws/barter/BarterWorkspaceWidget/view/".$bartid."/";
			
		$subject = Brick::ReplaceVarByData($v['cmtsubject'], array(
				"tl" => $bdBart['title']
		));
		$body = Brick::ReplaceVarByData($v['cmtbody'], array(
				"unm" => $this->manager->UserNameBuild($this->user->info),
				"plnk" => $plnk,
				"tl" => $bdBart['title'],
				"sitename" => Brick::$builder->phrase->Get('sys', 'site_name'),
				"cmt" => $data->bd." "
		));
		Abricos::Notify()->SendMail($email, $subject, $body);
	}
	
	public function MyBarterRemove($bartid){
		if (!$this->IsBarterWrite($bartid)){ return null; }
		$bart = $this->PrivateBarter($bartid);
		
		// удалить можно только те, что не публиковались
		if ($bart['pubdate']>0){ return null; }
	}
	
	public function MyBarterSave($p){
		if (!$this->manager->IsWriteRole()){ return null; }
		
		$p->id = intval($p->id);
		
		$utmf = Abricos::TextParser(true);
		$utm  = Abricos::TextParser(true);
		$utm->jevix->cfgSetAutoBrMode(true);
		
		if (!$this->manager->CategoryCheck($p->catid)){
			// попытки использовать не существующий каталог нужно пресекать
			sleep(1);
			return null;
		}
		
		$p->tl = trim($utmf->Parser($p->tl));
		$cdesc = trim($utmf->Parser($p->dsc));
		
		if (empty($p->tl) || empty($cdesc)){ return null; }
		
		$p->dsc = $utm->Parser($p->dsc);
		$p->dft = empty($p->dft) ? 0 : 1;
		
		if ($p->id == 0){
			$p->pdt = empty($p->dft) ? TIMENOW : 0;
			
			$p->id = JobBarterQuery::MyBarterAppend($this->db, $this->userid, $p);
		}else{
			if (!$this->IsBarterWrite($p->id)){ return null; }
			
			$bart = $this->PrivateBarter($p->id);
			$p->pdt = intval($bart['pubdate']);
			
			if (!empty($bart['isdraft']) && $p->dft == 0 && $p->pdt == 0){
				$p->pdt = TIMENOW;
			}
			JobBarterQuery::MyBarterUpdate($this->db, $this->userid, $p);
		}
		return $this->Barter($p->id);;
	}
	
	private $_bartCache = array();
	private function PrivateBarter($bartid){
		if (!empty($this->_bartCache[$bartid])){ return $this->_bartCache[$bartid]['bart']; }
		
		$this->_bartCache[$bartid] = array(
			"bart" => JobBarterQueryPrivate::Barter($this->db, $bartid)
		);
		return $this->_bartCache[$bartid]['bart'];
	}
	
	public function IsBarterWrite($bartid){
		if (!$this->manager->IsWriteRole()){ return false; }
		$p = $this->PrivateBarter($bartid);
		
		return $p['userid'] == $this->userid && empty($p['isspam']);
	}
}

?>