<?php
/**
 * @version $Id: manproject.php 1542 2012-05-03 05:39:49Z roosit $
 * @package Abricos
 * @subpackage Job
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin <roosit@abricos.org>
 */

require_once 'dbqproject.php';

/**
 * Управляющий класс модуля
 */
class JobProjectManager {
	
	/**
	 * @var JobManager
	 */
	public $manager = null;
	
	/**
	 * @var Ab_Database
	 */
	public $db = null;
	
	/**
	 * User
	 * @var User
	 */
	public $user = null;
	public $userid = 0;
	

	public function __construct(JobManager $manager){
		$this->manager = $manager;
		$this->user = $manager->user;
		$this->userid = $manager->userid;
		$this->db = $manager->db;
	}
	
	public function AJAX($d){
		switch($d->do){
			case 'projectlist':		return $this->ProjectList($d);
			case 'project':			return $this->Project($d->projid);
			case 'projectsave':		return $this->MyProjectSave($d);
			case 'answerappend':	return $this->AnswerAppend($d->projid, $d->answer);
		}
		return null;
	}
	
	public function CategoryProjectCount(){
		if (!$this->manager->IsViewRole()){ return null; }

		$rows = JobProjectQuery::CategoryProjectCount($this->db);
		
		$ret = array();
		while (($row = $this->db->fetch_array($rows))){
			$ret[$row['id']] = intval($row['c']);
		}
		return $ret;
	}
	
	public function ProjectList($cfg){
		if (!$this->manager->IsViewRole()){ return null; }
		
		$next = intval($cfg->next);
		$rows = JobProjectQuery::ProjectList($this->db, $this->userid, $cfg->catid, $next);
		
		$users = array();
		$ret = new stdClass();
		$ret->list = $this->manager->ToArray($rows, $users);
		
		if ($next == 0){
			$ret->pcnt = $this->CategoryProjectCount();
			$ret->fcnt = JobProjectQuery::ProjectListCount($this->db, $this->userid, $cfg->catid);
		}
		
		$rows = JobQuery::UserListByIds($this->db, $users);
		$ret->users = $this->manager->ToArray($rows);
		
		return $ret;
	}
	
	public function Project($projid){
		if (!$this->manager->IsViewRole() || empty($projid)){ return null; }

		$ret = new stdClass();

		$ret->p = JobProjectQuery::ProjectList($this->db, $this->userid, $catid, 0, $projid);
		$ret->pcnt = $this->CategoryProjectCount();
		
		$users = array();
		$ret->p['answers'] = $this->AnswerList($projid, $users);
		$users[$ret->p['uid']] = $ret->p['uid'] ;
		
		$rows = JobQuery::UserListByIds($this->db, $users);
		$ret->users = $this->manager->ToArray($rows);
		
		if ($this->userid > 0){
			JobProjectQuery::ProjectUserUpdateLastView($this->db, $projid, $this->userid);
		}
		
		return $ret;
	}
	
	public function AnswerList($projid, &$users = ""){
		if (!$this->manager->IsViewRole()){ return null; }
		$rows = JobProjectQuery::AnswerList($this->db, $projid, $this->userid);
		return $this->manager->ToArray($rows, $users);
	}
	
	public function AnswerAppend($projid, $ans){
		$ret = new stdClass();
		$ret->err  = null;
		$ret->ansid = 0;
		$ret->project = null;
		
		if (!$this->manager->IsWriteRole()){ 
			$ret->err = 1; 
			return $ret; 
		}
		
		$utmf = Abricos::TextParser(true);
		$utm  = Abricos::TextParser(true);
		$utm->jevix->cfgSetAutoBrMode(true);
		
		$cdesc = trim($utmf->Parser($ans->dsc));
		if (empty($cdesc)){
			$ret->err = 2; 
			return $ret; 
		}
		$ans->dsc = $utm->Parser($ans->dsc);
		
		// есть ли такой проект
		$bdProj = JobProjectQueryPrivate::Project($this->db, $projid);
		if (
				empty($bdProj) 
				|| $bdProj['deldate']>0 // проект удален 
				|| $bdProj['userid'] == $this->userid // ответ на своей проект
				|| $bdProj['isdraft']>0 // на черновик отвечать нельзя
			){
			$ret->err = 3;
			return $ret;
		}
		
		$bdAns = JobProjectQueryPrivate::Answer($this->db, $projid, $this->userid);
		if (!empty($bdAns)){ // можно оставить только одно предложение
			$ret->err = 4;
			return $ret;
		}
		
		$aid = JobProjectQuery::AnswerAppend($this->db, $projid, $this->userid, $ans);
		$ret->err  = $error;
		$ret->ansid = $aid;
		$ret->project = $this->Project($projid);
		
		JobProjectQuery::ProjectAnswerCountUpdate($this->db, $projid);
		
		// уведомить автора
		$autor = UserQuery::User($this->db, $bdProj['userid']);
		$email = $autor['email'];

		$brick = Brick::$builder->LoadBrickS('job', 'emailproject', null, null);
		$v = &$brick->param->var;
		$host = $_SERVER['HTTP_HOST'] ? $_SERVER['HTTP_HOST'] : $_ENV['HTTP_HOST'];
		$plnk = "http://".$host."/bos/#app=job/wspace/ws/project/ProjectWorkspaceWidget/view/".$projid."/";
		
		$subject = Brick::ReplaceVarByData($v['answersubject'], array(
			"tl" => $bdProj['title']
		));
		$body = Brick::ReplaceVarByData($v['answerbody'], array(
			"unm" => $this->manager->UserNameBuild($this->user->info),
			"plnk" => $plnk,
			"tl" => $bdProj['title'],
			"sitename" => Brick::$builder->phrase->Get('sys', 'site_name'),
			"anstext" => $ans->dsc
		));
		Abricos::Notify()->SendMail($email, $subject, $body);
		
		return $ret;
	}
	
	public function CommentSendNotify($ansInfo, $data){

		// данные по комментарию:
		// $data->id	- идентификатор комментария
		// $data->pid	- идентификатор родительского комментария
		// $data->uid	- пользователь оставивший комментарий
		// $data->bd	- текст комментария
		// $data->cid	- идентификатор контента
		
		JobProjectQuery::AnswerCommentCountUpdate($this->db, $data->cid);
		
		$projid = $ansInfo['pid'];
		
		$bdProj = JobProjectQueryPrivate::Project($this->db, $projid);
		$bdAns = JobProjectQueryPrivate::Answer($this->db, $projid, $ansInfo['userid']);
		
		// кому будет адресовано уведомление
		$userid = $this->userid == $bdProj['userid'] ? $bdAns['userid'] : $bdProj['userid'];
		$user = UserQuery::User($this->db, $userid);
		$email = $user['email'];
		
		$brick = Brick::$builder->LoadBrickS('job', 'emailproject', null, null);
		$v = &$brick->param->var;
		$host = $_SERVER['HTTP_HOST'] ? $_SERVER['HTTP_HOST'] : $_ENV['HTTP_HOST'];
		$plnk = "http://".$host."/bos/#app=job/wspace/ws/project/ProjectWorkspaceWidget/view/".$projid."/";
				
		$subject = Brick::ReplaceVarByData($v['cmtsubject'], array(
			"tl" => $bdProj['title']
		));
		$body = Brick::ReplaceVarByData($v['cmtbody'], array(
			"unm" => $this->manager->UserNameBuild($this->user->info),
			"plnk" => $plnk,
			"tl" => $bdProj['title'],
			"sitename" => Brick::$builder->phrase->Get('sys', 'site_name'),
			"cmt" => $data->bd." "
		));
		Abricos::Notify()->SendMail($email, $subject, $body);
	}
	
	public function MyProjectRemove($projid){
		if (!$this->IsProjectWrite($projid)){ return null; }
		$proj = $this->PrivateProject($projid);
		
		// удалить можно только те, что не публиковались
		if ($proj['pubdate']>0){ return null; }
	}
	
	public function MyProjectSave($p){
		if (!$this->manager->IsWriteRole()){ return null; }
		
		$p->id = intval($p->id);
		
		$utmf = Abricos::TextParser(true);
		$utm  = Abricos::TextParser(true);
		$utm->jevix->cfgSetAutoBrMode(true);
		
		if (!$this->manager->CategoryCheck($p->catid)){
			// попытки использовать не существующий каталог нужно пресекать
			sleep(1);
			return null;
		}
		
		$p->tl = trim($utmf->Parser($p->tl));
		$cdesc = trim($utmf->Parser($p->dsc));
		
		if (empty($p->tl) || empty($cdesc)){ return null; }
		
		$p->dsc = $utm->Parser($p->dsc);
		$p->dft = empty($p->dft) ? 0 : 1;
		
		$p->agr = empty($p->agr) ? 0 : 1;
		$p->cst = intval($p->cst);
		$p->cctp = intval($p->cctp);
		$p->pctp = intval($p->pctp);
		
		if ($p->cctp > 3 || $p->pctp > 4){ return null; }
		
		if ($p->agr){
			$p->cst = $p->cctp = $p->pctp = 0;
		}
		
		if ($p->id == 0){
			$p->pdt = empty($p->dft) ? TIMENOW : 0;
			
			$p->id = JobProjectQuery::MyProjectAppend($this->db, $this->userid, $p);
		}else{
			if (!$this->IsProjectWrite($p->id)){ return null; }
			
			$proj = $this->PrivateProject($p->id);
			$p->pdt = intval($proj['pubdate']);
			
			if (!empty($proj['isdraft']) && $p->dft == 0 && $p->pdt == 0){
				$p->pdt = TIMENOW;
			}
			JobProjectQuery::MyProjectUpdate($this->db, $this->userid, $p);
		}
		return $this->Project($p->id);
	}
	
	private $_projCache = array();
	private function PrivateProject($projid){
		if (!empty($this->_projCache[$projid])){ return $this->_projCache[$projid]['proj']; }
		
		$this->_projCache[$projid] = array(
			"proj" => JobProjectQueryPrivate::Project($this->db, $projid)
		);
		return $this->_projCache[$projid]['proj'];
	}
	
	public function IsProjectWrite($projid){
		if (!$this->manager->IsWriteRole()){ return false; }
		$p = $this->PrivateProject($projid);
		
		return $p['userid'] == $this->userid && empty($p['isspam']);
	}
}

?>