<?php
/**
 * Схема таблиц данного модуля.
 * 
 * @version $Id: shema.php 1548 2012-05-03 09:09:38Z roosit $
 * @package Abricos
 * @subpackage Job
 * @license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
 * @author Alexander Kuzmin <roosit@abricos.org>
 */

$charset = "CHARACTER SET 'utf8' COLLATE 'utf8_general_ci'";
$updateManager = Ab_UpdateManager::$current; 
$db = Abricos::$db;
$pfx = $db->prefix;

if ($updateManager->isInstall()){
	// инсталлировать роли модуля
	Abricos::GetModule('job')->permission->Install();

	// Таблица категорий
	$db->query_write("
		CREATE TABLE IF NOT EXISTS ".$pfx."job_category (
		  `categoryid` int(10) unsigned NOT NULL auto_increment COMMENT 'Идентификатор категории',
		  `parentcategoryid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор родителя',
		  `title` varchar(250) NOT NULL DEFAULT '' COMMENT 'Название',
		  `descript` TEXT NOT NULL  COMMENT 'Описание',

		  `onlyproject` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Только для черновиков',
		  `onlybarter` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Только для черновиков',
	
          `ordw` int(4) unsigned NOT NULL DEFAULT 0 COMMENT 'Сортировка. Чем больше, тем выше',
			
          `dateline` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время создания',
		  `deldate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время удаления',
		  PRIMARY KEY  (`categoryid`),
		  KEY `userid` (`parentcategoryid`)
		)".$charset
	);
	
	// Платные проекты
	$db->query_write("
		CREATE TABLE IF NOT EXISTS ".$pfx."job_project (
		  `projectid` int(10) unsigned NOT NULL auto_increment COMMENT 'Идентификатор проекта',
		  `categoryid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор категории',
		  `userid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор автора',
		  
		  `title` varchar(250) NOT NULL DEFAULT '' COMMENT 'Название',
		  `descript` TEXT NOT NULL  COMMENT 'Описание',

		  `agreement` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'По договоренности',
		  `cost` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Оплата',
		  `currencytype` int(2) unsigned NOT NULL DEFAULT 0 COMMENT 'Тип валюты',
		  `priceby` int(2) unsigned NOT NULL DEFAULT 0 COMMENT 'Цена за: 1-час, 2-день, 3-месяц, 4-проект',
	
		  `isdraft` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Сохранен в черновики',
          `pubdate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время публикации',

		  `isspam` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Модератор пометил как спам',
	
          `dateline` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время создания',
          `upddate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время изменения',
          `deldate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время удаления',
	
    	  `answercount` int(4) unsigned NOT NULL DEFAULT 0 COMMENT 'Кол-во предложений по проекту',
	
		  PRIMARY KEY  (`projectid`),
		  KEY `categoryid` (`categoryid`),
		  KEY `userid` (`userid`)
		  
		)".$charset
	);

	// Связь пользователя с проектом
	$db->query_write("
		CREATE TABLE IF NOT EXISTS ".$pfx."job_projectuser (
		  `projectid` int(10) unsigned NOT NULL COMMENT 'Идентификатор проекта',
		  `userid` int(10) unsigned NOT NULL COMMENT 'Идентификатор автора',
		  
          `dateline` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время создания записи',
          `lastview` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время последнего просмотра',
	
    	  `answercount` int(4) unsigned NOT NULL DEFAULT 0 COMMENT 'Кол-во предложений по проекту',
	
		  PRIMARY KEY  (`projectid`,`userid`)
		)".$charset
	);
	
	// Ответы на платные проекты
	$db->query_write("
		CREATE TABLE IF NOT EXISTS ".$pfx."job_panswer (
		  `panswerid` int(10) unsigned NOT NULL auto_increment COMMENT 'Идентификатор проекта',
		  `projectid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор проекта',
		  `contentid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор контента для комментария',
		  `userid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор автора',
		  
		  `bcost` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Бюджет от',
		  `ecost` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Бюджет до',
		  `currencytype` int(2) unsigned NOT NULL DEFAULT 0 COMMENT 'Тип валюты',

		  `btime` int(2) unsigned NOT NULL DEFAULT 0 COMMENT 'Время от',
		  `etime` int(2) unsigned NOT NULL DEFAULT 0 COMMENT 'Время до',
		  `ttime` int(1) unsigned NOT NULL DEFAULT 0 COMMENT '1-в часах, 2-в днях, 3-в месяцах',
	
		  `isspam` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Модератор пометил как спам',

 		  `isprivate` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Приватная задача',
	
          `dateline` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время создания',
          `upddate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время изменения',
          `deldate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время удаления',

       	  `cmtcount` int(4) unsigned NOT NULL DEFAULT 0 COMMENT 'Кол-во комментарий по проекту',
	
		  PRIMARY KEY  (`panswerid`),
		  KEY `projectid` (`projectid`),
		  UNIQUE KEY `answer` (`projectid`,`userid`)
		)".$charset
	);
	
	// Проекты по бартеру
	$db->query_write("
		CREATE TABLE IF NOT EXISTS ".$pfx."job_barter (
		  `barterid` int(10) unsigned NOT NULL auto_increment COMMENT 'Идентификатор проекта',
		  `categoryid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор категории',
		  `userid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор автора',
		  
		  `title` varchar(250) NOT NULL DEFAULT '' COMMENT 'Название',
		  `descript` TEXT NOT NULL  COMMENT 'Описание',

		  `agreement` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'По договоренности',
		  `cost` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Оплата',
		  `currencytype` int(2) unsigned NOT NULL DEFAULT 0 COMMENT 'Тип валюты',
		  `priceby` int(2) unsigned NOT NULL DEFAULT 0 COMMENT 'Цена за: 1-час, 2-день, 3-месяц, 4-проект',
	
		  `isdraft` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Сохранен в черновики',
          `pubdate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время публикации',

		  `isspam` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Модератор пометил как спам',
	
          `dateline` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время создания',
          `upddate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время изменения',
          `deldate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время удаления',
	
    	  `answercount` int(4) unsigned NOT NULL DEFAULT 0 COMMENT 'Кол-во предложений по проекту',
	
		  PRIMARY KEY  (`barterid`),
		  KEY `categoryid` (`categoryid`),
		  KEY `userid` (`userid`)
		  
		)".$charset
	);
	
	// Связь пользователя с проектом
	$db->query_write("
		CREATE TABLE IF NOT EXISTS ".$pfx."job_barteruser (
		  `barterid` int(10) unsigned NOT NULL COMMENT 'Идентификатор проекта',
		  `userid` int(10) unsigned NOT NULL COMMENT 'Идентификатор автора',
		  
          `dateline` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время создания записи',
          `lastview` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время последнего просмотра',
	
    	  `answercount` int(4) unsigned NOT NULL DEFAULT 0 COMMENT 'Кол-во предложений по проекту',
	
		  PRIMARY KEY  (`barterid`,`userid`)
		)".$charset
	);
	
	// Ответы на задачи по бартеру
	$db->query_write("
		CREATE TABLE IF NOT EXISTS ".$pfx."job_banswer (
		  `banswerid` int(10) unsigned NOT NULL auto_increment COMMENT 'Идентификатор ответа',
		  `barterid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор задачи',
		  `contentid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор контента для комментария',
		  `userid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор автора',

		  `btime` int(2) unsigned NOT NULL DEFAULT 0 COMMENT 'Время от',
		  `etime` int(2) unsigned NOT NULL DEFAULT 0 COMMENT 'Время до',
		  `ttime` int(1) unsigned NOT NULL DEFAULT 0 COMMENT '1-в часах, 2-в днях, 3-в месяцах',
	
		  `isspam` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Модератор пометил как спам',

 		  `isprivate` tinyint(1) unsigned NOT NULL DEFAULT 0 COMMENT 'Приватная задача',
	
          `dateline` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время создания',
          `upddate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время изменения',
          `deldate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время удаления',

       	  `cmtcount` int(4) unsigned NOT NULL DEFAULT 0 COMMENT 'Кол-во комментарий по проекту',
	
		  PRIMARY KEY  (`banswerid`),
		  KEY `barterid` (`barterid`),
		  UNIQUE KEY `answer` (`barterid`,`userid`)
		)".$charset
	);	
	
	// Проф. возможности для бартера задач
	$db->query_write("
		CREATE TABLE IF NOT EXISTS ".$pfx."job_scope (
		  `scopeid` int(10) unsigned NOT NULL auto_increment COMMENT 'Идентификатор знания',
		  `categoryid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор категории',
		  `userid` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Идентификатор пользователя',
		  `descript` TEXT NOT NULL  COMMENT 'Описание',
		  
          `dateline` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время создания',
          `upddate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время обновления',
          `deldate` int(10) unsigned NOT NULL DEFAULT 0 COMMENT 'Дата/время удаления',

		  PRIMARY KEY  (`scopeid`),
		  KEY `userid` (`userid`),
		  KEY `categoryid` (`categoryid`),
		  UNIQUE KEY `scope` (`userid`,`categoryid`)
		)".$charset
	);
}

if (!$updateManager->isInstall() && $updateManager->isUpdate('0.1.0.2')){
	$db->query_write("
		ALTER TABLE ".$pfx."job_category
			ADD `ordw` int(4) unsigned NOT NULL DEFAULT 0 COMMENT 'Сортировка. Чем больше, тем выше'
	");
}

?>