/*
@version $Id: categorylist.js 1542 2012-05-03 05:39:49Z roosit $
@package Abricos
@license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

var Component = new Brick.Component();
Component.requires = {
	mod:[
        {name: '{C#MODNAME}', files: ['lib.js']}
	]
};
Component.entryPoint = function(NS){
	var Dom = YAHOO.util.Dom,
		E = YAHOO.util.Event,
		L = YAHOO.lang;
	
	var CE = YAHOO.util.CustomEvent;
	
	var buildTemplate = this.buildTemplate;
	
	var CategorySelectWidget = function(container, selCategoryId, cfg){
		cfg = L.merge({
			'notgroup': false
		}, cfg || {});
		this.init(container, selCategoryId, cfg);
	};
	CategorySelectWidget.prototype = {
		init: function(container, selCategoryId, cfg){
			
			this._selectedValue = 0;
			this.container = container;
			this.list = NS.jobManager.categoryList;
			this.cfg = cfg;
			
			buildTemplate(this, 'selwidget,selrow,selrowchild,selrown,selrownchild');
			
			var __self = this;
			E.on(container, 'click', function(e){
				var el = E.getTarget(e);
				if (__self.onClick(el)){ E.preventDefault(e); }
			});
			
			this.render();
			this.selectCategory(selCategoryId);
			
			NS.jobManager.categoryChangedEvent.subscribe(this.onCategoryChanged, this, true);
		},
		destroy: function(){
			NS.jobManager.categoryChangedEvent.unsubscribe(this.onCategoryChanged);
		},
		onCategoryChanged: function(){
			this.render();
		},
		render: function(){
			var TM = this._TM, lst = "", __self = this;
			
			if (this.cfg['notgroup']){
				this.list.foreach(function(cat){
					lst += TM.replace('selrown', {
						'id': cat.id,
						'tl': cat.title
					});
					cat.foreach(function(ccat){
						lst += TM.replace('selrownchild', {
							'id': ccat.id,
							'tl': ccat.title
						});
						
					});
				});
			}else{
				this.list.foreach(function(cat){
					lst += TM.replace('selrow', {
						'id': cat.id,
						'tl': cat.title,
						'list': __self.buildChildList(cat)
					});
				});
			}

			this.container.innerHTML = TM.replace('selwidget', {
				'list': lst
			});
			this.setValue(this._selectedValue);
		},
		buildChildList: function(cat){
			var TM  = this._TM, lst = "";
			cat.foreach(function(ccat){
				lst += TM.replace('selrowchild', {'id': ccat.id, 'tl': ccat.title });
			});
			return lst;
		},
		onClick: function(el){
			return false;
		},
		selectCategory: function(catid){
			this._selectedValue = catid;
			this._TM.getEl('selwidget.id').value = catid;
		},
		setValue: function(catid){
			this.selectCategory(catid);
		},
		getValue: function(){
			return this._TM.getEl('selwidget.id').value;
		}
	};
	NS.CategorySelectWidget = CategorySelectWidget;
	
	var _globalSavedSelCatid = 0;
	
	var CategoryColListWidget = function(container, selCategoryId, cfg){
		selCategoryId = (selCategoryId || 0) * 1;
		cfg = L.merge({
			'tcount': 'project'
		}, cfg || {});
		this.init(container, selCategoryId, cfg);
	};
	CategoryColListWidget.prototype = {
		init: function(container, selCategoryId, cfg){
			this.config = cfg;
			this.list = NS.jobManager.categoryList;
			
			buildTemplate(this, 'clwidget,clrow,clrowchild');
			container.innerHTML = this._TM.replace('clwidget', {
				'list': this.buildList()
			});
			this.updateCount();
			NS.jobManager.categoryChangedEvent.subscribe(this.reBuildList, this, true);
			NS.jobManager.categoryCountChangedEvent.subscribe(this.updateCount, this, true);
			
			var __self = this;
			E.on(container, 'click', function(e){
				var el = E.getTarget(e);
				if (__self.onClick(el)){ E.preventDefault(e); }
			});
			
			this.selectedId = 0;
			this.selectCategory(selCategoryId);
			
			this.selectChangedEvent = new CE('selectChangedEvent');
			this.clickItemEvent = new CE('clickItemEvent');
			
			if (selCategoryId == 0 && _globalSavedSelCatid > 0){
				this.showChildList(_globalSavedSelCatid);
			}
		},
		destroy: function(){
			NS.jobManager.categoryChangedEvent.unsubscribe(this.reBuildList);
			NS.jobManager.categoryCountChangedEvent.unsubscribe(this.updateCount);
		},
		onSelectChange: function(){
			if (this.selectChangedEvent){
				this.selectChangedEvent.fire();
			}
		},
		updateCount: function(){
			var TId = this._TId,
				tcount = this.config['tcount'];
			
			NS.jobManager.categoryForeach(function(ccat){
				var el = Dom.get(TId['clrowchild']['cnt']+'-'+ccat.id);
				if (L.isNull(el)){ return; }

				var cnt = '';
				if (tcount == 'project' && ccat.projCount > 0){
					cnt = ccat.projCount;
				}
				el.innerHTML = cnt;
			}, true);
		},
		buildChildList: function(cat){
			var TM  = this._TM, lst = "";
			
			cat.foreach(function(ccat){
				lst += TM.replace('clrowchild', {
					'id': ccat.id,
					'cnt': '',
					'tl': ccat.title
				});
			});
			
			return lst;
		},
		buildList: function(){
			var TM  = this._TM, lst = "", __self = this;
			this.list.foreach(function(cat){
				lst += TM.replace('clrow', {
					'id': cat.id,
					'tl': cat.title,
					'list': __self.buildChildList(cat)
				});
			});
			return lst;
		},
		reBuildList: function(){
			var TM = this._TM;
			TM.getEl('clwidget.list').innerHTML = this.buildList();
			
			this.updateCount();
			this.selectCategory(this.selectedId);
			this.showChildList(_globalSavedSelCatid);
		},
		onClick: function(el){
			var TId = this._TId,
				prefix = el.id.replace(/([0-9]+$)/, ''),
				numid = el.id.replace(prefix, "");
			
			switch(prefix){
			case (TId['clrow']['title']+'-'):
				if (!this.onClickItem(numid)){
					this.showChildList(numid); 
				}
				return true;
			case (TId['clrowchild']['id']+'-'):
				if (!this.onClickItem(numid)){
					this.selectCategory(numid);
				}
				return true;
			}

			return false;
		},
		onClickItem: function(catid){
			this.clickItemEvent.fire(catid);
			return false;
		},
		showChildList: function(catid){
			var TId = this._TId;
			this.list.foreach(function(cat){
				var el = Dom.get(TId['clrow']['child']+'-'+cat.id);
				Dom.setStyle(el, 'display', catid == cat.id ? '' : 'none');
			});
			_globalSavedSelCatid = catid;
		},
		selectCategory: function(catid){
			// if (this.selectedId == catid){ return; }
			this.selectedId = catid;
			
			var TId = this._TId, pcatid = 0;
			this.list.foreach(function(cat){
				cat.foreach(function(ccat){
					var el = Dom.get(TId['clrowchild']['id']+'-'+ccat.id);
					
					if (catid == ccat.id){
						Dom.addClass(el, 'sel');
						pcatid = ccat.parentid;
					}else{
						Dom.removeClass(el, 'sel');
					}
				});
			});
			if (pcatid > 0){
				this.showChildList(pcatid);
			}
			this.onSelectChange();
		},
		setValue: function(catid){
			this.selectCategory(catid);
		},
		getValue: function(){
			return this.selectedId;
		}
	};
	NS.CategoryColListWidget = CategoryColListWidget;
	
};