/*
@version $Id: projeditor.js 1542 2012-05-03 05:39:49Z roosit $
@package Abricos
@license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

var Component = new Brick.Component();
Component.requires = {
	mod:[
		{name: '{C#MODNAME}', files: ['lib.js']}
	]
};
Component.entryPoint = function(NS){
	
	var Dom = YAHOO.util.Dom,
		E = YAHOO.util.Event,
		L = YAHOO.lang;
	
	var R = NS.roles;
	
	var LNG = Brick.util.Language.geta(['mod', '{C#MODNAME}']);
	
	var buildTemplate = this.buildTemplate;
	
	var ProjectEditorWidget = function(container, project, callback){
		this.init(container, project, callback);
	};
	ProjectEditorWidget.prototype = {
		init: function(container, proj, callback){
			this.project = proj;
			this.callback = callback;

			buildTemplate(this, 'widget');
			var __self = this;
			var TM = this._TM,
				gel = function(nm){ return TM.getEl('widget.'+nm); };
				
			container.innerHTML = TM.replace('widget', {
				'disnew': proj.id > 0 ? 'none' : '',
				'disedit': proj.id > 0 ? '' : 'none',
				'isdraft': proj.isdraft ? '' : 'none',
				'isndraft': !proj.isdraft ? '' : 'none'
			});
			
			this.category = new NS.CategorySelectWidget(gel('catalog'), proj.catid);
			gel('title').value = proj.title;
			gel('descript').value = NS.textToEdit(proj.descript);
			gel('agreement').checked = proj.agreement ? 'checked' : '';
			gel('cost').value = proj.cost > 0 ? proj.cost : '';
			gel('currency').value = proj.currency;
			gel('priceby').value = proj.priceby;
			
			if (proj.id == 0){
				Dom.setStyle(gel('bpublic'), 'display', '');
				Dom.setStyle(gel('bpublicsave'), 'display', 'none');
			}
			
			this.updateAgreementStatus();
			E.on(gel('agreement'), 'change', function(){
				__self.updateAgreementStatus();
			});
			
			E.on(container, 'click', function(e){
				var el = E.getTarget(e);
				if (__self.onClick(el)){ E.preventDefault(e); }
			});
		},
		updateAgreementStatus: function(){
			var TM = this._TM,
				gel = function(nm){ return TM.getEl('widget.'+nm); };
			
			var dis = gel('agreement').checked ? 'disabled' : '';
			gel('cost').disabled = dis;
			gel('currency').disabled = dis;
			gel('priceby').disabled = dis;
		},
		destroy: function(){
			var el = this._TM.getEl('widget.id');
			el.parentNode.removeChild(el);
		},
		onClick: function(el){
			this.errorHide();
			var tp = this._TId['widget'];
			switch(el.id){
			case tp['bpublic']: 
			case tp['bpublicsave']: 
				this.projectPublic(); return true;
			case tp['bdraft']: 
			case tp['bdraftsave']: 
				this.projectDraft(); return true;
			case tp['bcancel']: this.cancel(); return true;
			}
			return false;
		},
		errorHide: function(){
			for (var i=1;i<7;i++){
				Dom.setStyle(this._TM.getEl('widget.err'+i), 'display', 'none');
			}
		},
		errorShow: function(err){
			this.errorHide();
			Dom.setStyle(this._TM.getEl('widget.err'+err), 'display', '');
		},
		cancel: function(){
			this.callback('cancel');
			this.destroy();
		},
		getSaveData: function(){
			var TM = this._TM,
				gel = function(nm){ return TM.getEl('widget.'+nm); };

			var d = {
				'id': this.project.id,
				'catid': this.category.getValue(),
				'tl': L.trim(gel('title').value),
				'dsc': L.trim(gel('descript').value),
				'agr': gel('agreement').checked ? 1 : 0,
				'cst': gel('cost').value*1,
				'cctp': gel('currency').value*1,
				'pctp': gel('priceby').value*1
			};

			var cdsc = L.trim(d['dsc'].replace(/<p>/gi, '').replace(/<\/p>/gi, ''));
			var err = 0;
			if (d['catid'] == 0){ err = 1;
			}else if (d['tl'].length < 5){ err = 2;
			}else if (cdsc.length < 10){ err = 3;
			}else if (d['agr'] == 0){
				if (d['cst']<1){ err = 4;
				}else if (d['cctp'] == 0){ err = 5;
				}else if (d['pctp'] == 0){ err = 6;
				}
			}
			
			if (err > 0){
				this.errorShow(err);
				return null;
			}
			return d;
		},
		projectPublic: function(){
			var sd = this.getSaveData();
			if (L.isNull(sd)){ return ; }
			sd['dft'] = 0;
			this.save(sd);
		},
		projectDraft: function(){
			var sd = this.getSaveData();
			if (L.isNull(sd)){ return ; }
			sd['dft'] = 1;
			this.save(sd);
		},
		save: function(sd){
			
			var TM = this._TM, 
				__self = this,
				proj = this.project;
			
			Dom.setStyle(TM.getEl('widget.btns'), 'display', 'none');
			Dom.setStyle(TM.getEl('widget.prcsave'), 'display', '');
			
			NS.projectManager.projectSave(sd, function(d){
				if (L.isNull(d)){
					__self.cancel();
				}else{
					proj.update(d['p']);
					__self.callback('save');
					__self.destroy();
				}
			});
		}
	};
	NS.ProjectEditorWidget = ProjectEditorWidget;
};