/*
@version $Id: projview.js 1542 2012-05-03 05:39:49Z roosit $
@package Abricos
@license http://www.gnu.org/copyleft/gpl.html GNU/GPL, see LICENSE.php
*/

var Component = new Brick.Component();
Component.requires = {
	mod:[
		{name: '{C#MODNAME}', files: ['projlib.js', 'projeditor.js', 'categorylist.js']}
	]
};
Component.entryPoint = function(NS){
	
	var Dom = YAHOO.util.Dom,
		E = YAHOO.util.Event,
		L = YAHOO.lang;
	
	var UID = Brick.env.user.id*1;
	
	var LNG = Brick.util.Language.geta(['mod', '{C#MODNAME}']);
	
	var buildTemplate = this.buildTemplate;

	var ProjectRowWidget = function(container, proj){
		this.init(container, proj);
	};
	ProjectRowWidget.prototype = {
		init: function(container, proj){
			this.project = proj;
			buildTemplate(this, 'row,rowview,rowcost,rowcostagr,bedit,rowcat');
			
			container.innerHTML += this._TM.replace('row');
			this.render();
			this.editorWidget = null;
		},
		destroy: function(){
			if (!L.isNull(this.editorWidget)){
				this.editorWidget.destroy();
			}
			var el = this._TM.getEl('row.id');
			el.parentNode.removeChild(el);
		},
		onClick: function(el){
			var tp = this._TId['bedit'];
			switch(el.id){
			case tp['bedit']: this.showEditorWidget(); return true;
			}
			return false;
		},
		render: function(){
			var TM = this._TM,
				proj = this.project,
				cat = NS.jobManager.categoryFind(proj.catid);

			TM.getEl('row.view').innerHTML = TM.replace('rowview', {
				'id': proj.id,
				'anscnt': proj.answerCount,
				'uriview': NS.navigator.project.view(proj.id),
				'title': proj.title,
				'cost': proj.agreement ? TM.replace('rowcostagr') : TM.replace('rowcost', {
					'val': NS.formatCost(proj.cost, proj.currency),
					'pctp': LNG['priceby']['vp'][proj.priceby]
				}),
				'bedit': Brick.env.user.id != proj.userid ? '' : TM.replace('bedit'),
				'pdt': proj.pubdate>0 && !proj.isdraft ? Brick.dateExt.convert(proj.pubdate) : LNG['draft'],
				'cat': L.isNull(cat) ? '' : TM.replace('rowcat', {'path': cat.path}),
				'desc': proj.descript
			});
		},
		showEditorWidget: function(){
			if (!L.isNull(this.editorWidget)){ return; }
			
			var __self = this,
				proj = this.project;
			
			var callback = function(tp, sd){
				__self.hideEditorWidget();
				if (tp == 'cancel'){
				}else if (tp == 'save'){
					__self.render();
				}
			};
			this.editorWidget = new NS.ProjectEditorWidget(this._TM.getEl('row.editor'), proj, callback);
			Dom.setStyle(this._TM.getEl('row.view'), 'display', 'none');
		},
		hideEditorWidget: function(){
			this.editorWidget = null;
			Dom.setStyle(this._TM.getEl('row.view'), 'display', '');
		}
	};
	NS.ProjectRowWidget = ProjectRowWidget;
	
	var ProjectInfoWidget = function(container, proj){
		this.init(container, proj);
	};
	ProjectInfoWidget.prototype = {
		init: function(container, proj){
			var TM = buildTemplate(this, 'pinfo,user');
			var user = NS.jobManager.users.get(proj.userid);
			container.innerHTML = TM.replace('pinfo',{
				'user': TM.replace('user', {
					'uid': user.id,
					'unm':  user.getUserName()
				})
			});
		},
		destroy: function(){},
		onClick: function(el){
			return false;
		}
	};
	NS.ProjectInfoWidget = ProjectInfoWidget;
	
	var PAnswerEditWidget = function(container, answer, callback){
		this.init(container, answer, callback);
	};
	PAnswerEditWidget.prototype = {
		init: function(container, answer, callback){
			this.answer = answer;
			this.callback = callback;
			
			var TM = buildTemplate(this, 'ansedit'),
				gel = function(n){ return TM.getEl('ansedit.'+n); };
			container.innerHTML = TM.replace('ansedit');
		},
		destroy: function(){},
		onClick: function(el){
			if (el.id == this._TId['ansedit']['bsave']){
				this.save();
				return true;
			}
			return false;
		},
		save: function(){
			if (!L.isFunction(this.callback)){ return; }
			var TM = this._TM, gel = function(n){ return TM.getEl('ansedit.'+n); };
			Dom.setStyle(gel('bsave'), 'display', 'none');
			Dom.setStyle(gel('saveproc'), 'display', '');
			this.callback({
				'bcost': gel('bcost').value,
				'ecost': gel('ecost').value,
				'ctp': gel('currency').value,
				'btm': gel('btime').value,
				'etm': gel('etime').value,
				'ttm': gel('ttime').value,
				'dsc':  L.trim(gel('descript').value),
				'prv': gel('private').checked ? 1 : 0
			});
		}
	};
	NS.PAnswerEditWidget = PAnswerEditWidget;
	
	var PAnswerRowWidget = function(container, answer){
		this.init(container, answer);
	};
	PAnswerRowWidget.prototype = {
		init: function(container, ans){
			this.answer = ans;
			var TM = buildTemplate(this, 'ansrow,user,ansrowcost,ansrowtime,ansb,anse'),
				gel = function(n){ return TM.getEl('ansrow.'+n); };

			var user = NS.jobManager.users.get(ans.userid);
			var sCost = "", sTime = "";
			if (ans.bCost>0 || ans.eCost>0){
				sCost = TM.replace('ansrowcost', {
					'p': (ans.bCost==0 ? '' : TM.replace('ansb', {'v': ans.bCost}))
					 + (ans.eCost==0 ? '' : TM.replace('anse', {'v': ans.eCost})),
					 'tp':  LNG['currency']['vp'][ans.tCost]
				});
			}

			if (ans.bTime>0 || ans.eTime>0){
				sTime = TM.replace('ansrowtime', {
					'p': (ans.bTime==0 ? '' : TM.replace('ansb', {'v': ans.bTime}))
					 + (ans.eTime==0 ? '' : TM.replace('anse', {'v': ans.eTime})),
					 'tp':  LNG['priceby']['vp'][ans.tTime]
				});
			}
			
			container.innerHTML += TM.replace('ansrow', {
				'avatar': user.avatar45(),
				'cost': sCost, 'time': sTime,
				'user': TM.replace('user', {
					'uid': user.id,
					'unm':  user.getUserName()
				}),
				'dl': Brick.dateExt.convert(ans.date)
			});
			gel('dsc').innerHTML = ans.descript;
			this.renderCommentCount();
		},
		destroy: function(){
			var el = this._TM.getEl('ansrow.id');
			el.parentNode.removeChild(el);
			this.answer = null;
		},
		renderCommentCount: function(){
			this._TM.getEl('ansrow.ccnt').innerHTML = this.answer.commentCount;
		},
		enableComment: function(){
			Dom.removeClass(this._TM.getEl('ansrow.cmtbox'), 'hide');
		},
		onClick: function(el){
			var tp = this._TId['ansrow'];
			switch(el.id){
			case tp['bshow']: this.showCommentBox(); return true;
			case tp['bhide']: this.hideCommentBox(); return true;
			}
			return false;
		},
		hideCommentBox: function(){
			Dom.replaceClass(this._TM.getEl('ansrow.cmtbox'), 'pashst', 'pahdst');
		},
		showCommentBox: function(){
			var ans = this.answer,
				TM = this._TM,
				gel = function(n){ return TM.getEl('ansrow.'+n); };
			
			Dom.replaceClass(gel('cmtbox'), 'pahdst', 'pashst');
			
			if (this._startld){ return; }
			this._startld = true;
			
			// Инициализировать менеджер комментариев
			Brick.ff('comment', 'comment', function(){
				Brick.mod.comment.API.buildCommentTree({
					'container': gel('cmt'),
					'dbContentId': ans.commentid,
					'config': {
						'onLoadComments': function(){
							// aTargetBlank(TM.getEl('panel.taskbody'));
							// aTargetBlank(TM.getEl('panel.comments'));
						}
						// ,
						// 'readOnly': project.w*1 == 0,
						// 'manBlock': L.isFunction(config['buildManBlock']) ? config.buildManBlock() : null
					},
					'instanceCallback': function(b){ }
				});
			});
			
		}
	};
	NS.PAnswerRowWidget = PAnswerRowWidget;
	
	var PAnswerListWidget = function(container, project){
		this.init(container, project);
	};
	PAnswerListWidget.prototype = {
		init: function(container, project){
			this.project = project;
			var TM = buildTemplate(this, 'anslist,ansrowempty');
			container.innerHTML = TM.replace('anslist');
			
			this.elList = TM.getEl('anslist.id');
			
			this.ws = [];
			this.render();
		},
		destroy: function(){
			this.clearws();
		},
		clearws: function(){
			for (var i=0;i<this.ws.length;i++){
				this.ws[i].destroy();
			}
			this.ws = [];
			this.elList.innerHTML = "";
		},
		renderRow: function(ans){
			var p = this.project,
				w = new NS.PAnswerRowWidget(this.elList, ans);
			
			if (UID == p.userid || UID == ans.userid){
				w.enableComment();
			}
			
			this.ws[this.ws.length] = w;
		},
		render: function(){
			this.clearws();
			var __self = this;
			this.project.answers.foreach(function(ans){
				__self.renderRow(ans);
			});
			if (this.ws.length == 0){
				this.elList.innerHTML = this._TM.replace('ansrowempty');
			}
		},
		onClick: function(el){
			for (var i=0;i<this.ws.length;i++){
				this.ws[i].onClick(el);
			}
			return false;
		}
	};
	NS.PAnswerListWidget = PAnswerListWidget;
	
	
	var ProjectViewWidget = function(container, projid, callback){
		this.init(container, projid, callback);
	};
	ProjectViewWidget.prototype = {
		init: function(container, projid, callback){
			var TM = buildTemplate(this, 'widget'), __self = this;
			container.innerHTML = TM.replace('widget');

			this.ws = {};
			
			NS.projectManager.projectLoad(projid, function(proj){
				__self.onLoadProject(proj);
				if (L.isFunction(callback)){
					callback(proj);
				}
			});
		},
		destroy: function(){
			for (var n in this.ws){
				this.ws[n].destroy();
			}
		},
		onLoadProject: function(proj){
			this.project = proj;
			var ws = this.ws, __self = this, 
				TM = this._TM, gel = function(n){ return TM.getEl('widget.'+n); };
			
			ws['projrow'] = new NS.ProjectRowWidget(gel('proj'), proj);
			ws['projinfo'] = new NS.ProjectInfoWidget(gel('pinfo'), proj);
			
			var userid = Brick.env.user.id*1,
				showNewAnswer = proj.userid != userid;
			
			if (showNewAnswer){
				proj.answers.foreach(function(ans){
					if (ans.userid == userid){
						showNewAnswer = false;
						return;
					}
				});
			}
			
			if (showNewAnswer){
				ws['ansnew'] = new NS.PAnswerEditWidget(gel('ansedit'), new NS.PAnswer(), function(d){
					__self.onAnswerAppend(d);
				});
			}
			ws['anslist'] = new NS.PAnswerListWidget(gel('anslist'), proj);

			E.on(TM.getEl('widget.id'), 'click', function(e){
				var el = E.getTarget(e);
				if (__self.onClick(el)){ E.preventDefault(e); }
			});
		},
		onClick: function(el){
			for (var n in this.ws){
				if (this.ws[n].onClick(el)){
					return true;
				}
			}
			return false;
		},
		onAnswerAppend: function(sd){
			var __self = this, TM = this._TM, ws = this.ws;
			
			NS.projectManager.answerAppend(this.project.id, sd, function(r){
				Dom.setStyle(TM.getEl('widget.ansedit'), 'display', 'none');
				ws['anslist'].render();
			});
		}
	};
	NS.ProjectViewWidget = ProjectViewWidget;
	
};